// -----------------------------------------------------------------------------
//
// TFileTypeRegistration-Klasse
// Copyright (c) 2004 Mathias Simmack
//
// -----------------------------------------------------------------------------

// -- Revision history ---------------------------------------------------------
//
//   * erste Version
//   * "FInternalName" in "GetInternalKey" zurckgesetzt (damit wird
//     u.U. ein Fehler vermieden, wenn der gesuchte Schlssel nicht
//     existiert, "FInternalName" aber noch von einer frheren Aktion
//     einen Wert enthielt)
//   * Explorer-Integration (Rechtsklick -> Neu -> ...) durch die zwei
//     Funktionen "AddNewFileSupport" und "RemoveNewFileSupport" mglich
//   * nderung in "AddNewFileSupport" und "RemoveNewFileSupport"
//     (Parameter entfernt und durch Property "FExtension" ersetzt)
//
// -----------------------------------------------------------------------------
unit filetypes;

interface

uses
  Windows, SysUtils, Registry, ShlObj,uisAdmin;

type
  TFileTypeRegistration = class
    FRegConnector : TRegistry;
    FExtension,
    FInternalName : string;
    FVerb         : string;
    function GetInternalValue(const KeyName, ValueName: string): string;
  public
    constructor Create;
    destructor Destroy; override;
    function RegisterType(const Extension, InternalName: string;
      Description: string = ''; IconFile: string = '';
      IconIndex: integer = -1): boolean;
    function UnregisterExtension(const Extension: string): boolean;
    function UnregisterType(const Extension: string): boolean;
    procedure UpdateShell;
    function AddHandler(const HandlerVerb, CommandLine: string;
      HandlerDescription: string = ''): boolean; overload;
    function DeleteHandler(const HandlerVerb: string): boolean;
    function SetDefaultHandler: boolean; overload;
    function SetDefaultHandler(const HandlerVerb: string): boolean; overload;
    function GetInternalKey(const Extension: string): string;
    function AddNewFileSupport: boolean;
    function RemoveNewFileSupport: boolean;

    property Extension: string read FExtension;
    property InternalName: string read FInternalName;
    property CurrentVerb: string read FVerb;
  end;


implementation

(* *****************************************************************************
  Beispiel #1: Einen neuen Dateityp registrieren
  ----------------------------------------------
  ftr := TFileTypeRegistration.Create;
  if(ftr <> nil) then
  try
    // die Dateiendung ".foo" registrieren, der interne Schlssel
    // lautet "FooFile", eine Beschreibung und eine Symboldatei
    // sind ebenfalls angegeben
    if(ftr.RegisterType('.foo','FooFile','FOO Description',
      'c:\folder\icon.ico')) then
    begin
      // fgt den Handler "open" hinzu und verknpft ihn mit dem
      // Programm "foo.exe"
      ftr.AddHandler('open','"c:\folder\foo.exe" "%1"');
      // setzt den zuletzt benutzten Handler ("open" in dem Fall)
      // als Standard
      ftr.SetDefaultHandler;
    end;
    if(ftr.RegisterType('.foo','ThisIsNotTheFOOKey')) then
    // Das ist kein Fehler! Obwohl hier der interne Name
    // "ThisIsNotTheFOOKey" verwendet wird, benutzt die Funktion
    // intern den bereits vorhandenen Schlssel "FooFile" (s. oben).
    begin
      // zwei neue Handler werden registriert, ...
      ftr.AddHandler('print','"c:\folder\foo.exe" /p "%1"');
      ftr.AddHandler('edit','notepad.exe "%1"');   
      // ... & dank der berladenen Funktion "SetDefaultHandler"
      // kann diesmal auch "print" als Standardhandler gesetzt
      // werden
      ftr.SetDefaultHandler('print');
    end;
  finally
    ftr.Free;
  end;
  Beispiel #2: Einen neuen Typ mit einem vorhandenen Schlssel
  verknpfen
  ------------------------------------------------------------
  Das Beispiel registriert die Endung ".foo" auf die gleiche
  Weise wie Textdateien (.txt). Es wird einfach der interne
  Schlsselname ermittelt und fr die Endung ".foo" gesetzt
  ftr := TFileTypeRegistration.Create;
  if(ftr <> nil) then
  try
    strInternalTextFileKey := ftr.GetInternalKey('.txt');
    if(strInternalTextFileKey <> '') then
      ftr.RegisterType('.foo',strInternalTextFileKey);
  finally
    ftr.Free;
  end;
  Beispiel #3: Einen Handler entfernen
  ------------------------------------
  ftr := TFileTypeRegistration.Create;
  if(ftr <> nil) then
  try
    // den internen Schlsselnamen des Typs ".foo" ermitteln, ...
    if(ftr.GetInternalKey('.foo') <> '') then
    // ... wobei das Ergebnis in dem Fall unwichtig ist, weil
    // intern auch die Eigenschaft "FInternalName" gesetzt
    // wird
    begin
      // den "print"-Handler entfernen, ...
      ftr.DeleteHandler('print');
      // ... & den Standardhandler aktualisieren
      ftr.SetDefaultHandler('open');
    end;
  finally
    ftr.Free;
  end;
  Beispiel #4: Nur eine Dateiendung entfernen
  -------------------------------------------
  In diesem Fall wird lediglich die Endung ".foo" entfernt. Der
  evtl. vorhandene interne Schlssel bleibt bestehen. Das ist
  fr das Beispiel #2 ntzlich, wenn die Endung ".foo" entfernt
  werden soll, intern aber mit den Textdateien verlinkt ist, die
  ja im Normalfall nicht entfernt werden drfen/sollten.
    ftr.UnregisterExtension('.foo');
  Beispiel #5: Den kompletten Dateityp entfernen
  ----------------------------------------------
  Dieses Beispiel entfernt dagegen den kompletten Dateityp,
  inkl. des evtl. vorhandenen internen Schlssels (vgl. mit
  Beispiel #4).
    ftr.UnregisterType('.foo');
  Bezogen auf Beispiel #2 wre das die fatale Lsung, weil dadurch
  zwar die Endung ".foo" deregistriert wird, gleichzeitig wird
  aber auch der intern verwendete Schlssel der Textdateien
  gelscht.
  ALSO, VORSICHT!!!
***************************************************************************** *)


//
// Admin-Rechte sind erforderlich (Funktion von NicoDE)
//


// -----------------------------------------------------------------------------
//
// TFileTypeRegistration-Klasse
//
// -----------------------------------------------------------------------------

constructor TFileTypeRegistration.Create;
begin
 
  FExtension    := '';
  FInternalName := '';
  FVerb         := '';

  // Zugriff auf die Registry, ...
  if(IsAdmin) then FRegConnector := TRegistry.Create
    else FRegConnector := nil;

  // ... & HKEY_CLASSES_ROOT als Root setzen
  if(FRegConnector <> nil) then
    FRegConnector.RootKey := HKEY_CLASSES_ROOT;
end;

destructor TFileTypeRegistration.Destroy;
begin
  if(FRegConnector <> nil) then
    FreeAndNil(FRegConnector);
end;

function TFileTypeRegistration.RegisterType(const Extension,
  InternalName: string; Description: string = ''; IconFile: string = '';
  IconIndex: integer = -1): boolean;
var
  strDummy : string;
begin
  // Standardergebnis
  Result         := false;
  if(FRegConnector = nil) or
    (Extension = '') or
    (Extension[1] <> '.') then exit;

  // ist dieser Typ evtl. schon registriert?
  strDummy := self.GetInternalKey(Extension);

  // Nein
  if(strDummy = '') then strDummy := InternalName;

  // den Schlssel mit der Dateiendung anlegen oder aktualisieren
  if(FRegConnector.OpenKey(Extension,true)) then
  try
    FRegConnector.WriteString('',strDummy);
    Result := (FRegConnector.ReadString('') = strDummy);
  finally
    FRegConnector.CloseKey;
  end;
  if(not Result) then exit;

  // den internen Schlssel ffnen
  if(Result) and
    (FRegConnector.OpenKey(strDummy,true)) then
  try
    // Beschreibung anlegen
    if(Description <> '') then
      FRegConnector.WriteString('',Description);

    // Symbol zuweisen (Datei muss existieren!)
    if(IconFile <> '') and
      (fileexists(IconFile)) and
      (FRegConnector.OpenKey('DefaultIcon',true)) then
    try
      if(IconIndex <> -1) then
        FRegConnector.WriteString('',Format('%s,%d',[IconFile,IconIndex]))
      else
        FRegConnector.WriteString('',IconFile);
    finally
      FRegConnector.CloseKey;
    end;
  finally
    FRegConnector.CloseKey;
  end;

  // Systemsymbole aktualisieren
  self.UpdateShell;

  // Properties aktualisieren
  if(Result) then
  begin
    FExtension    := Extension;
    FInternalName := strDummy;
  end;
end;

function TFileTypeRegistration.UnregisterExtension(const Extension: string):
  boolean;
begin
  Result := false;
  if(FRegConnector = nil) or
    (Extension = '') or
    (Extension[1] <> '.') then exit;

  // die Endung entfernen
  Result := (FRegConnector.KeyExists(Extension)) and
    (FRegConnector.DeleteKey(Extension));

  // Systemsymbole aktualisieren
  self.UpdateShell;
end;

function TFileTypeRegistration.UnregisterType(const Extension: string):
  boolean;
var
  strDummy : string;
begin
  Result   := false;
  if(FRegConnector = nil) or
    (Extension = '') or
    (Extension[1] <> '.') then exit;

  // den internen Namen der Endung ermitteln
  strDummy := self.GetInternalKey(Extension);

  // die Endung entfernen (s. "UnregisterExtension"), ...
  Result   := (self.UnregisterExtension(Extension)) and
  // ... & den internen Schlssel lschen
    (strDummy <> '') and
    (FRegConnector.KeyExists(strDummy)) and
    (FRegConnector.DeleteKey(strDummy));

  // Systemsymbole aktualisieren
  self.UpdateShell;
end;

procedure TFileTypeRegistration.UpdateShell;
begin
  SHChangeNotify(SHCNE_ASSOCCHANGED,SHCNF_IDLIST,nil,nil);
end;

const
  ShellKey = '%s\shell\%s';

function TFileTypeRegistration.AddHandler(const HandlerVerb,
  CommandLine: string; HandlerDescription: string = ''): boolean;
begin
  // Standardergebnis
  Result := false;
  if(FRegConnector = nil) or
    (FInternalName = '') or
    (HandlerVerb = '') or
    (CommandLine = '') then exit;

  // der interne Schlssel muss existieren
  if(FRegConnector.KeyExists(FInternalName)) then
  begin
    // den Handler (= Verb) erzeugen
    if(FRegConnector.OpenKey(Format(ShellKey + '\command',
      [FInternalName,HandlerVerb]),true)) then
    try
      FRegConnector.WriteString('',CommandLine);
      Result := (FRegConnector.ReadString('') = CommandLine);
    finally
      FRegConnector.CloseKey;
    end;

    // ggf. Beschreibung fr Handler setzen
    if(HandlerDescription <> '') then
    begin
      if(FRegConnector.OpenKey(Format(ShellKey,[FInternalName,
        HandlerVerb]),true)) then
      try
        FRegConnector.WriteString('',HandlerDescription);
      finally
        FRegConnector.CloseKey;
      end;
    end;
  end;

  // interne Eigenschaft anpassen (fr "SetDefaultHandler")
  if(Result) then
    FVerb := HandlerVerb;
end;

function TFileTypeRegistration.DeleteHandler(const HandlerVerb: string):
  boolean;
begin
  // Standardergebnis
  Result := false;
  if(FRegConnector = nil) or
    (FInternalName = '') or
    (HandlerVerb = '') then exit;

  // Handlerschlssel entfernen (sofern vorhanden)
  Result :=
    (FRegConnector.KeyExists(Format(ShellKey,[FInternalName,HandlerVerb]))) and
    (FRegConnector.DeleteKey(Format(ShellKey,[FInternalName,HandlerVerb])));
end;

function TFileTypeRegistration.SetDefaultHandler: boolean;
begin
  if(FInternalName <> '') and (FVerb <> '') then
    Result := self.SetDefaultHandler(FVerb)
  else
    Result := false;
end;

function TFileTypeRegistration.SetDefaultHandler(const HandlerVerb: string):
  boolean;
begin
  Result := false;
  if(FRegConnector = nil) or
    (FInternalName = '') or
    (HandlerVerb = '') then exit;

  // interner Schlssel muss existieren, ...
  if(FRegConnector.KeyExists(FInternalName)) and
  // ... & Handler muss existieren, ...
    (FRegConnector.KeyExists(Format(ShellKey,[FInternalName,HandlerVerb]))) and
    (FRegConnector.OpenKey(FInternalName + '\shell',false)) then
  try
  // ... dann den Handler als Standard eintragen
    FRegConnector.WriteString('',HandlerVerb);
    Result := (FRegConnector.ReadString('') = HandlerVerb);
  finally
    FRegConnector.CloseKey;
  end;
end;

function TFileTypeRegistration.GetInternalKey(const Extension: string): string;
begin
  if(FRegConnector = nil) or
    (Extension = '') or
    (Extension[1] <> '.') then exit;

  // einen evtl. eingestellten internen Namen zurcksetzen
  FInternalName   := '';

  // den Schlssel der Dateiendung ffnen, ...
  if(FRegConnector.KeyExists(Extension)) and
    (FRegConnector.OpenKey(Extension,false)) then
  try
  // ... & den Standardwert auslesen, ...
    FInternalName := FRegConnector.ReadString('');
  finally
    FRegConnector.CloseKey;
  end;

  // ... als Funktionsergebnis zurckliefern
  if(not FRegConnector.KeyExists(FInternalName)) then
    FInternalName := '';

  Result := FInternalName;
end;

function TFileTypeRegistration.GetInternalValue(const KeyName,
  ValueName: string): string;
begin
  Result := '';
  if(FInternalName = '') then exit;

  if(FRegConnector.KeyExists(FInternalName)) and
    (FRegConnector.OpenKey(Format('%s\%s',[FInternalName,KeyName]),false)) then
  try
    Result := FRegConnector.ReadString(ValueName);
  finally
    FRegConnector.CloseKey;
  end;
end;


function TFileTypeRegistration.AddNewFileSupport: boolean;
var
  Description : string;
begin
  Result      := false;
  if(FRegConnector = nil) or
    (FExtension = '') or
    (FExtension[1] <> '.') then exit;

  // interne Beschreibung des Typs ermitteln
  if(self.GetInternalKey(FExtension) <> '') then
    Description := self.GetInternalValue('','')
  else
    Description := '';

  // die Beschreibung darf keine Leerzeichen enthalten, weil sie
  // als Referenz fr den neuen Dateinamen verwendet wird, ...
  if(pos(#32,Description) > 0) or
  // ... & sie darf auch nicht leer sein
    (Description = '') then exit;

  // den Eintrag fr die Explorer-Integration anlegen
  if(FRegConnector.KeyExists(FExtension)) and
    (FRegConnector.OpenKey(FExtension + '\ShellNew',true)) then
  try
    FRegConnector.WriteString('NullFile','');
    Result := (FRegConnector.ReadString('NullFile') = '');
  finally
    FRegConnector.CloseKey;
  end;
end;

function TFileTypeRegistration.RemoveNewFileSupport:
  boolean;
begin
  Result := false;
  if(FRegConnector = nil) or
    (FExtension = '') or
    (FExtension[1] <> '.') then exit;

  Result := (FRegConnector.KeyExists(FExtension + '\ShellNew')) and
    (FRegConnector.DeleteKey(FExtension + '\ShellNew'));
end;

end.
